import {Component, Input, OnInit, ViewChild} from '@angular/core';
import {Activity, CarePlan} from '@nspop/gm-web-facade-api';
import {ModalComponent} from '../../modals/modal/modal.component';
import {ActivitiesService} from '../activities.service';
import {copy} from '../../../utils/copy';
import {CarePlanService} from '../../api/care-plan.service';

@Component({
  selector: 'app-edit-plan-modal',
  templateUrl: './edit-plan-modal.component.html',
  styleUrls: ['./edit-plan-modal.component.scss']
})
export class EditPlanModalComponent implements OnInit {

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @Input() public selectedActivities: boolean[] = [];

  public carePlan: CarePlan | undefined;
  public activities: Activity[] = [];
  // public additionalActivities: Activity[] = []; // unused since backend does not support this. Check .html for the commented section.
  public editableActivities: number[] = []; // Array of indexes

  constructor(
    public activitiesService: ActivitiesService,
    public carePlanService: CarePlanService,
  ) {
  }

  ngOnInit(): void {
  }

  // public editActivity(activity: Activity): void { // unused since backend does not (yet?) support additionalActivties.
  //   this.activitiesService.emitOpenActivityModalEvent(activity);
  // }

  public setVisible(isVisible: boolean): void {
    this.carePlan = this.activitiesService.carePlan;

    this.editableActivities = [];
    this.carePlan?.activities?.forEach((activity: Activity, index: number) => {
      if (activity.ownership !== 'otherUser' && activity.status !== 'finished') {
        this.editableActivities.push(index);
      }
    });

    // Initialize selectedActivities. Checked activities (= true) are those whose status that aren't cancelled.
    this.selectedActivities = this.carePlan?.activities?.map((activity: Activity) => activity.status !== 'cancelled') ?? [];
    this.modalComponent?.setVisible(isVisible);
  }

  isVisible(): boolean {
    return !!this.modalComponent?.isModalVisible;
  }

  public submit(): void {
    if (!this.carePlan?.activities) { return; }
    const oldCarePlan = copy(this.carePlan); // Take a copy for reverting purposes

    this.carePlan.activities.forEach((activity: Activity, i: number) => {
      this.setActivityStatus(activity, i); // Update all activity statuses
    });

    this.submitCarePlan(this.carePlan, oldCarePlan);
  }

  private submitCarePlan(carePlan: CarePlan, oldCarePlan: CarePlan): void {
    this.carePlanService.update(carePlan, carePlan.id + '', carePlan.version + '').subscribe(
      () => {
        this.modalComponent?.setVisible(false);
        this.carePlanService.refresh();
      },
      () => {
        this.carePlan = oldCarePlan;
      });
  }

  private setActivityStatus(activity: Activity, i: number): void {
    const isNotAllowedToChange = !this.editableActivities.includes(i);
    if (isNotAllowedToChange) { return; }

    const isNotSelected = !this.selectedActivities[i];
    const isCancelled = activity.status === 'cancelled';

    if (isNotSelected) {
      activity.status = 'cancelled';
    } else if (isCancelled) {
      activity.status = 'expected';
    }
  }

  toggleAll(): void {
    const allSelected = copy(!this.allSelected());
    this.selectedActivities = this.selectedActivities.map((isSelected: boolean, index: number) => {
      if (!this.editableActivities.includes(index)) { return isSelected; }
      return allSelected;
      }
    );
  }

  public someButNotAllSelected(): boolean {
    const sumSelected = this.editableActivities.map((a: number) => this.selectedActivities[a]).filter((a: boolean) => a).length;
    return sumSelected > 0 && sumSelected < this.editableActivities.length;
  }

  public allSelected(): boolean {
    const sumSelected = this.editableActivities.map((a: number) => this.selectedActivities[a]).filter((a: boolean) => a).length;
    return sumSelected === this.editableActivities.length;
  }
}
