import {Component, ElementRef, HostListener, OnInit, ViewChild} from '@angular/core';
import {
  ChronicDiseasesSection,
  DataCard,
  OverallAssessmentSectionAllOfPrenatalCareAssessment,
  PosNegEnum,
} from '@nspop/gm-web-facade-api';
import ConditionsEnum = ChronicDiseasesSection.ConditionsEnum;
import CareLevelEnum = OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum;
import {DataCardService} from '../api/data-card.service';

@Component({
  selector: 'app-general-information-bar',
  templateUrl: './general-information-bar.component.html',
  styleUrls: ['./general-information-bar.component.scss']
})
export class GeneralInformationBarComponent implements OnInit {
  @ViewChild('headerContainer') public headerContainer: ElementRef | undefined;
  public headerInScrollMode = false;

  public dataCard: DataCard | undefined;
  public chronicDiseases = 'Nej';
  public careLevel = '?';
  public rhesusType = '?';
  constructor(private dataCardService: DataCardService) { }

  ngOnInit(): void {
    this.dataCardService.activeDataCard$
      .subscribe((dataCard: DataCard | undefined) => {
        this.dataCard = dataCard;
        this.chronicDiseases = this.computeSpecialConditions(
          this.dataCard?.chronicDiseasesSection
          || {dirty: false} as ChronicDiseasesSection); // TODO: Is missing modifiedTime and author!!
        this.careLevel = this.computeCareLevel(this.dataCard?.overallAssessmentSection?.prenatalCareAssessment?.careLevel);
        this.rhesusType = this.computeRhesusType(this.dataCard?.screeningsSection?.maternalRhDAntibody?.rhesusType);
      });
  }

  get expectedDateOfBirth(): {route: string, params: any} {
    if (this.dataCard?.expectedDateOfDeliverySection?.expectedDateOfDeliveryFromPrenatalUltrasoundScan) {
      return {
        route: '/undersøgelser-og-målinger',
        params: {tab: 1, highlight: 'birth-date-ultrasound-birth-date'}
      };
    }
    return {
      route: '/svangerskab',
      params: {tab: 1, highlight: 'terminsberegning'}
    };
  }

  public swapEncounter(): void {
    console.log('Swapping encounter..');
  }

  public finishEncounter(): void {
    console.log('Finish encounter..');
  }

  private computeCareLevel(carelevel?: CareLevelEnum): string {
    switch (carelevel) {
      case CareLevelEnum.Basic:
        return '1';
      case CareLevelEnum.Extended:
        return '2';
      case CareLevelEnum.SpecialistMultidisciplinaryTeam:
        return '3';
      case CareLevelEnum.HighRiskPregnancyCare:
        return '4';
      default:
        return '?';
    }
  }

  private computeRhesusType(type?: PosNegEnum): string {
    switch (type) {
      case 'positive':
        return 'Positiv';
      case 'negative':
        return 'Negativ';
      default:
        return '?';
    }
  }

  private computeSpecialConditions(chronicDiseasesSection: ChronicDiseasesSection): string {
    if (!chronicDiseasesSection.conditions){
      return 'nej';
    }
    return chronicDiseasesSection.conditions.map(
      key => {
        switch (key) {
          case ConditionsEnum.CirculatorySystem:
            return 'Kredsløbssygdomme';
          case ConditionsEnum.RespiratorySystem:
            return 'Åndedrætssystem';
          case ConditionsEnum.Thyroid:
            return 'Skjoldbruskkirtel';
          case ConditionsEnum.Diabetes:
            return 'Diabetes';
          default:
            return '';
        }}
    ).join(', ');
  }

  @HostListener('window:scroll', ['$event']) // for window scroll events
  public onScroll($event: Event): void {
    if (this.headerContainer) {
      const elemTop = this.headerContainer.nativeElement.getBoundingClientRect().top;
      this.headerInScrollMode = elemTop <= 0;
    }
  }

}
