import {ComponentFixture, TestBed} from '@angular/core/testing';

import {EditUltrasoundComponent} from './edit-ultrasound.component';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {AppModule} from '../../../app.module';
import {FormsModule, NgForm} from '@angular/forms';
import {DataCardWrapper} from '../../../model-wrappers/dataCard.wrapper';

describe('EditUltrasoundComponent', () => {
  let component: EditUltrasoundComponent;
  let fixture: ComponentFixture<EditUltrasoundComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ EditUltrasoundComponent ],
      imports: [MatIconTestingModule, AppModule, FormsModule],
      providers: [
        {provide: NgForm, useValue: new NgForm([], [])},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(EditUltrasoundComponent);
    component = fixture.componentInstance;
    component.observationDataUltrasound = {
      fetuses: [{}, {}]
    } as any;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('adds fetuses if number of fetuses in data-card increases', () => {
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(3);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(3);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  it('does not remove fetuses if data-card has fetus-count 0', () => {
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(0);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(2);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  it('does not remove fetuses if data-card has no fetus-count', () => {
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(undefined);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(2);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  it('removes fetuses if number if fetuses in data-card descreases', () => {
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(1);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(1);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  function changeFetusCount(count?: number): void {
    component.dataCard = new DataCardWrapper({
      expectedDateOfDeliverySection: {},
      prenatalRiskAssessmentSection: {
        finalRiskAssessment: {
          multiplicityRiskAssessment: {}
        },
      },
      overallAssessmentSection: {prenatalCareAssessment: {numberOfFetuses: count}}
    } as any);
    component.ngOnChanges({dataCard: {}} as any);
    fixture.detectChanges();
  }
});
