import {Component, OnInit, ViewChild} from '@angular/core';
import {DataCard, DataCardAllOfScreeningsSection} from '@nspop/gm-web-facade-api';
import {finalize} from 'rxjs/operators';
import {NgForm} from '@angular/forms';
import {FormComponent} from '../../confirm-navigation-if-dirty.framework';
import {DataCardService} from '../../api/data-card.service';

@Component({
  selector: 'app-measurements-tab',
  templateUrl: './measurements-tab.component.html',
  styleUrls: ['./measurements-tab.component.scss'],
})
export class MeasurementsTabComponent extends FormComponent implements OnInit{
  public dataCard: DataCard | undefined;
  public screenings: DataCardAllOfScreeningsSection | undefined;
  public disableSaveButton = false;
  public ultraSoundSaveListeners: (() => any)[] = [];
  private identifier = '';
  private version = '';
  public viewActivityDialog: {isOpen: boolean; activityId?: string} = {isOpen: false};

  public editUltrasoundDialog: {isOpen: boolean; observationType: 'regular' | 'ultrasound' } = {isOpen: false, observationType: 'regular'};
  @ViewChild('form', {static: true}) form: NgForm|undefined;

  constructor(private dataCardService: DataCardService) {
    super();
    (window as any).form = this.form;
  }

  ngOnInit(): void {
    this.dataCardService.activeDataCard$.subscribe((dataCard?: DataCard) => {
      if (dataCard) {
        this.identifier = dataCard.documentUniqueId || '';
        this.version = dataCard.version || '';
        this.dataCard = dataCard;
        this.screenings = dataCard.screeningsSection;
      }
    });
  }

  onUltrasoundSave(): void {
    this.ultraSoundSaveListeners.forEach( f => f());
  }

  public save(): void {
    if (this.form?.valid && this.dataCard) {
      this.disableSaveButton = true;
      this.dataCardService.update(this.dataCard, this.identifier, this.version)
        .pipe(finalize(() => this.disableSaveButton = false))
        .subscribe(() => {
          this.dataCardService.refresh();
        });
    }
  }
}
