import {fakeAsync, TestBed, tick} from '@angular/core/testing';

import { DialogService } from './dialog.service';
import {Component} from '@angular/core';
import {MatDialogModule, MatDialogRef} from '@angular/material/dialog';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';

@Component({
  selector: 'app-test-dialog',
  template: '<h1 mat-dialog-title id="TestDialogService">TEST DIALOG</h1>' +
      '<div mat-dialog-content>' +
        'HERES THE CONTENT' +
      '</div>' +
      '<div mat-dialog-actions>' +
        '<i>HERES THE ACTIONS</i>' +
      '</div>'
})
class TestDialogComponent {}

describe('DialogService', () => {
  let service: DialogService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      declarations: [TestDialogComponent],
      imports: [MatDialogModule, BrowserAnimationsModule]
    });
    service = TestBed.inject(DialogService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('Opens dialog', fakeAsync(() => {
    service.show(TestDialogComponent);
    tick();
    expect(document.body.innerHTML).toContain('TEST DIALOG');
    expect(document.body.innerHTML).toContain('HERES THE CONTENT');
    expect(document.body.innerHTML).toContain('HERES THE ACTIONS');
  }));

  it('closes an open dialog', fakeAsync(() => {
    service.show(TestDialogComponent).subscribe((value) => {
      expect(value).toEqual('my value');
    });
    tick();
    expect(document.body.innerHTML).toContain('TEST DIALOG');
    expect(document.body.innerHTML).toContain('HERES THE CONTENT');
    expect(document.body.innerHTML).toContain('HERES THE ACTIONS');
    const dialogRef = Object.values((service as any).dialogs)[0] as MatDialogRef<any>;
    const materialCloseMethod = spyOn(dialogRef, 'close').and.callThrough();
    service.close(dialogRef.componentInstance, 'my value');
    expect(materialCloseMethod).toHaveBeenCalledWith('my value');
  }));

  it('errors when trying to close a doalog which is not open', () => {
    expect(() => service.close({})).toThrowError('Could not close dialog - did you try to close it without opening it?');
    expect(() => service.close({dialog_service_identifier: 'whatever'})).toThrowError('Could not close dialog - did you try to close it without opening it?');
  });
});
