import {NgModule} from '@angular/core';
import {Route, RouterModule, Routes} from '@angular/router';
import {MeasurementsTabComponent} from '../measurements/measurements-tab/measurements-tab.component';
import {ActivitiesComponent} from '../activities/activities.component';
import {PregnancyComponent} from '../pregnancy/pregnancy.component';
import {PatientInformationComponent} from '../patient-information/patient-information.component';
import {TriageOverviewComponent} from '../triage-overview/triage-overview.component';
import {PrivatelyMarkedComponent} from '../privately-marked/privately-marked.component';
import {NegatePrivatelyMarkedGuard} from '../privately-marked/negate-privately-marked.guard';
import {SomethingWentWrongComponent} from '../something-went-wrong/something-went-wrong.component';
import {appUrls} from '../../utils/URL_STRING_LITERALS';
import {ComponentType} from '@angular/cdk/portal';
import {PrivatelyMarkedGuard} from '../privately-marked/privately-marked.guard';
import {DirtyGuard} from '../confirm-navigation-if-dirty.framework';

/**
 * The application routing parameters
 */
export const appRoutes: Routes = [
  route(appUrls.triageOverview, TriageOverviewComponent),
  route(appUrls.activities, ActivitiesComponent),
  route(appUrls.measurements, MeasurementsTabComponent),
  route(appUrls.pregnancy, PregnancyComponent),
  route(appUrls.patientInformation, PatientInformationComponent),
  route(appUrls.privatelyMarked, PrivatelyMarkedComponent, [NegatePrivatelyMarkedGuard]),
  route(appUrls.somethingWentWrong, SomethingWentWrongComponent),
  { path: appUrls.base, redirectTo: appUrls.activities}, // Default/wildcard redirect
];

function route(path: string, component?: ComponentType<any>, canActivate?: ComponentType<any>[]): Route {
  return {
    path,
    component,
    canActivate: canActivate || [PrivatelyMarkedGuard],
    canDeactivate: [DirtyGuard],
  };
}

@NgModule({
  imports: [RouterModule.forRoot(appRoutes, {useHash: true})],
  exports: [RouterModule]
})

export class RoutingModule { }
