import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {PatientInformationComponent} from './patient-information.component';
import {of} from 'rxjs';
import {Component, Directive, Input} from '@angular/core';
import {delay} from 'rxjs/operators';
import {FormsModule} from '@angular/forms';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {CardComponent} from '../card/card.component';
import {NamePipe} from './name.pipe';
import {OptionalDatePipe} from './optional-date.pipe';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {MatMenuModule} from '@angular/material/menu';
import {MatIconModule} from '@angular/material/icon';
import {MaskableInputComponent} from '../maskable-input/maskable-input.component';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {PersonalDataSectionAllOfNextOfKin, YesNoEnum} from '@nspop/gm-web-facade-api';
import RelationEnum = PersonalDataSectionAllOfNextOfKin.RelationEnum;
import {AppModule} from '../app.module';
import {DataCardService} from '../api/data-card.service';

@Component({
  selector: 'app-radio-group',
  template: '<ng-content></ng-content>'
})
class MockGroupWrapperComponent {}

@Component({
  selector: 'app-general-information-bar',
  template: '<p>information-bar</p>'
})
class MockGeneralInformationBarComponent {}

@Component({
  selector: 'app-semi-fixed-header',
  template: '<ng-content></ng-content>'
})
class MockSemiFixedHeaderComponent {
  // tslint:disable-next-line:no-input-rename
  @Input('fixed-at') fixedAt = 0;
}

// tslint:disable-next-line:directive-selector
@Directive({selector: 'mask'})
class MaskMockDirective{}

describe('PatientInformationComponent', () => {
  let component: PatientInformationComponent;
  let fixture: ComponentFixture<PatientInformationComponent>;

  let dataCardServiceStub: DataCardService;

  beforeEach(async () => {
    dataCardServiceStub = {
      update: () => of({}).pipe(
        delay(100)
      ),
      refresh: () => {},
      activeDataCard$: of({
        dirty: false,
        episodeOfCareIdentifier : '1123',
        documentUniqueId: '1',
        version: 'asdasd',
        documentStatus: 'ACTIVE',
        recordTarget: {
          identifier: '1008821234',
          name: {
            firstNames: ['Peter'],
            middleNames: [],
            lastName: 'Parker'
          }
        },
        socialHistorySection: {
          dirty: false,
          partner: {
            translationLanguage: 'something',
            nationality: 'whetever',
            needsTranslation: YesNoEnum.True
          }
        },
        expectedDateOfDeliverySection: {
          dirty: false,
          expectedDateOfDeliveryFromLastPeriod: {},
          trimester: 3,
          monthsIntoPregnancy: 2
        }
      })
    } as any;

    await TestBed.configureTestingModule({
      declarations: [
        PatientInformationComponent,
        MockGroupWrapperComponent,
        MockGeneralInformationBarComponent,
        NamePipe,
        OptionalDatePipe,
        MaskableInputComponent,
        MockSemiFixedHeaderComponent,
        MaskMockDirective,
        CardComponent
      ],
      imports: [
        AngularMaterialModule,
        AppModule,
        FormsModule,
        NoopAnimationsModule,
        MatCheckboxModule,
        MatMenuModule,
        MatIconModule
      ],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub}
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PatientInformationComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
    expect((component as unknown as {documentUniqueId: string}).documentUniqueId).toEqual('1');
  });

  it('disables save button while saving', fakeAsync(() => {
    expect(component.disableSaveButton).toBeFalse();
    component.save();
    expect(component.disableSaveButton).toBeTrue();
    tick(200);
    expect(component.disableSaveButton).toBeFalse();
  }));

  // it('translates relations to next of kin', () => {
  //   Object.values(RelationEnum).forEach((value: string) => {
  //     console.log('NextOfKin value:', value);
  //     expect(component.displayRelation(value)).not.toBe('?');
  //     expect(component.displayRelation(value)).not.toBe(value);
  //   });
  // });

  it('translates unknown relations to next to "?"', () => {
    expect(component.displayRelation('an_unreasonable_code')).toBe('?');
    // The next two cases should never happen, but we don't want to crash if they do
    expect(component.displayRelation(null as unknown as string)).toBe('?');
    expect(component.displayRelation(undefined as unknown as string)).toBe('?');
  });

  it('handles that active datacard could be undefined', fakeAsync(() => {
    component = new PatientInformationComponent({activeDataCard$: of()} as any);
    component.ngOnInit();
    tick();
    expect(component.dataCard).toBeUndefined();
  }));
});
