import {Component, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {Subscription} from 'rxjs';
import {DataCard} from '@nspop/gm-web-facade-api';
import {NgForm} from '@angular/forms';
import {FormComponent} from '../confirm-navigation-if-dirty.framework';
import {copy} from '../../utils/copy';
import {DataCardService} from '../api/data-card.service';

@Component({
  selector: 'app-patient-information',
  templateUrl: './patient-information.component.html',
  styleUrls: ['./patient-information.component.scss']
})
export class PatientInformationComponent extends FormComponent implements OnInit, OnDestroy {
  @ViewChild('form', {static: true}) form: NgForm | undefined;
  public dataCard: DataCard | undefined;
  private documentUniqueId = '';
  private version = '';
  private subscription: Subscription | undefined;
  public disableSaveButton = false;

  constructor(private dataCardService: DataCardService) {
    super();
  }

  ngOnInit(): void {
    this.subscription = this.dataCardService.activeDataCard$
      .subscribe((dataCard?: DataCard) => {
        if (dataCard) {
          this.documentUniqueId = dataCard.documentUniqueId || '';
          this.version = dataCard.version || '';
          this.dataCard = copy(dataCard);
        }
      });
  }

  ngOnDestroy(): void {
    this.subscription?.unsubscribe();
  }

  public displayRelation(relation: string): string {
    switch (relation) {
      case 'uspec_paaroerende':
        return 'Ukendt';
      case 'ingen_relationer':
        return 'Ingen relation';
      case 'barn':
        return 'Barn';
      case 'mor':
        return 'Mor';
      case 'far':
        return 'Far';
      case 'forælder':
        return 'Forælder';
      case 'søskende':
        return 'Søskende';
      case 'aegtefaelle':
        return 'Ægtefælle';
      case 'barnebarn':
        return 'Barnebarn';
      case 'svigerbarn':
        return 'Svigerbarn';
      case 'øvrig_familie':
        return 'Øvrig familie';
      case 'nabo':
        return 'Nabo';
      case 'samboende':
        return 'Samboende';
      case 'registreret_partner':
        return 'Registreret partner';
      case 'anden':
        return 'Anden';
      case 'søn':
        return 'Søn';
      case 'datter':
        return 'Datter';
      default:
        return '?';
    }
  }

  public save(): void {
    this.disableSaveButton = true;
    this.dataCardService.update(this.dataCard as DataCard, this.documentUniqueId, this.version)
      .subscribe(() => {
        this.disableSaveButton = false;
        this.dataCardService.refresh();
      });
  }
}
