import { ComponentFixture, TestBed } from '@angular/core/testing';

import { SemiFixedHeaderComponent } from './semi-fixed-header.component';

describe('SemiFixedHeaderComponent', () => {
  let component: SemiFixedHeaderComponent;
  let fixture: ComponentFixture<SemiFixedHeaderComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ SemiFixedHeaderComponent ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(SemiFixedHeaderComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('it enters scroll-mode when window is scrolled', (done) => {
    spyOn(component.headerContainer?.nativeElement, 'getBoundingClientRect').and.returnValue({top: -200});
    expect(component.headerInScrollMode).toBeFalse();
    window.dispatchEvent(new Event('scroll'));
    fixture.detectChanges();
    fixture.whenStable().then(() => {
      expect(component.headerInScrollMode).toBeTrue();
      done();
    });
  });

  it('it leaves scroll-mode when window is scrolled back', (done) => {
    // First let's scroll down
    const returnValue = {top: -200};
    spyOn(component.headerContainer?.nativeElement, 'getBoundingClientRect').and.returnValue(returnValue);
    expect(component.headerInScrollMode).toBeFalse();
    window.dispatchEvent(new Event('scroll'));
    fixture.detectChanges();

    // Cool... Now let's scroll back up
    returnValue.top = 10;
    window.dispatchEvent(new Event('scroll'));
    fixture.detectChanges();

    // Now we'd expect to no longer be in scroll mode
    fixture.whenStable().then(() => {
      expect(component.headerInScrollMode).toBeFalse();
      done();
    });
  });

  it('it does not enter scroll-mode when window is scrolled as long as it\'s not scrolled past fixed-at', (done) => {
    component.fixedAt = -201;
    spyOn(component.headerContainer?.nativeElement, 'getBoundingClientRect').and.returnValue({top: -200});
    expect(component.headerInScrollMode).toBeFalse();
    window.dispatchEvent(new Event('scroll'));
    fixture.detectChanges();
    fixture.whenStable().then(() => {
      expect(component.headerInScrollMode).toBeFalse();
      done();
    });
  });

});
