import { Injectable } from '@angular/core';
import {Observable, Subject} from 'rxjs';
import {environment} from '../../environments/environment';

@Injectable({
  providedIn: 'root'
})
export class ActiveRequests {
  private blockingRequestCount = new Subject<number>();
  private blockingRequests = 0;
  constructor() { }

  public countBlockingRequests(): Observable<number> {
    return this.blockingRequestCount.asObservable();
  }

  get hasBlockingRequests(): boolean {
    return !!this.blockingRequests;
  }

  public _registerBlockingRequest(): void {
    this.blockingRequestCount.next(++this.blockingRequests);
  }

  public _unregisterBlockingRequest(): void {
    if (this.blockingRequests - 1 < 0) { // We're decreasing below 0 active requests... something is wrong
      if (environment.production) { // Let's not hard-crash in production...
        console.warn('Cannot decrease number of active requests below 0. Loading mask may behave strangely from now on :(');
        return;
      }
      throw new Error('Cannot decrease number of active requests below 0'); // In dev mode we want to know if this happens
    } else {
      this.blockingRequestCount.next(--this.blockingRequests);
    }
  }

}
