import {fakeAsync, flush, TestBed} from '@angular/core/testing';

import {PatientContextService} from './patient-context.service';
import {WindowService} from './window.service';

describe('PatientContextService', () => {
  let service: PatientContextService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        {provider: WindowService, useValue: {nativeWindow: () => ({
              document: {cookie: ''},
              localStorage: {getItem: () => null, setItem: () => undefined}}
            )}
        }
      ]
    });
    service = TestBed.inject(PatientContextService);
  });

  beforeEach(fakeAsync(() => {
    service.setCpr('1234567890');
    service.setEpisodeOfCare('69');
    service.setXBreakTheGlassReason('stangspring');
    service.setHasPrivatelyMarkedData(false);
    flush();
  }));

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('gets patient synchronously', () => {
    service.setCpr('0987654321');
    expect(service.getCurrentCpr()).toBe('0987654321');
  });

  it('gets episodeOfCare synchronously', () => {
    service.setEpisodeOfCare('777777');
    expect(service.getCurrentEpisodeOfCare()).toBe('777777');
  });

  it('gets xBreakTheGlassReason synchronously', () => {
    service.setXBreakTheGlassReason('værdispring');
    expect(service.getCurrentXBreakTheGlassReason()).toBe('værdispring');
  });

  it('gets hasPrivatelyMarkedData synchronously', () => {
    service.setHasPrivatelyMarkedData(true);
    expect(service.getCurrentHasPrivatelyMarkedData()).toBe(true);
  });

  describe('when episodeOfCare changes', () => {
    it('triggers observable when episodeOfCare changes', fakeAsync(() => {
      let calledTimes = 0;
      const sub = service.episodeOfCare$.subscribe((episode) => {
        expect(episode).toBe('4');
        calledTimes++;
      });
      service.setEpisodeOfCare('4');
      flush();
      expect(calledTimes).toBe(1);
      sub.unsubscribe();
    }));
  });

  describe('when xBreakTheGlassReason changes', () => {
    it('triggers observable when xBreakTheReason changes', fakeAsync(() => {
      let calledTimes = 0;
      const sub = service.hasPrivatelyMarkedData$.subscribe((value) => {
        expect(value).toBe(true);
        calledTimes++;
      });
      service.setHasPrivatelyMarkedData(true);
      flush();
      expect(calledTimes).toBe(1);
      sub.unsubscribe();
    }));
  });

  describe('when xBreakTheGlassReason changes', () => {
    it('triggers observable when xBreakTheReason changes', fakeAsync(() => {
      let calledTimes = 0;
      const sub = service.xBreakTheGlassReason$.subscribe((reason) => {
        expect(reason).toBe('hestespring');
        calledTimes++;
      });
      service.setXBreakTheGlassReason('hestespring');
      flush();
      expect(calledTimes).toBe(1);
      sub.unsubscribe();
    }));
  });

  describe('when patient changes', () => {
    it('triggers observable', fakeAsync(() => {
      let calledTimes = 0;
      const sub = service.cpr$.subscribe((cpr) => {
        expect(cpr).toBe('0987654321');
        calledTimes++;
      });
      service.setCpr('0987654321');
      flush();
      expect(calledTimes).toBe(1);
      sub.unsubscribe();
    }));

    it('clears episodeOfCare', fakeAsync(() => {
      let calledTimes = 0;
      const sub = service.episodeOfCare$.subscribe((episode) => {
        expect(episode).toBeUndefined();
        calledTimes++;
      });
      service.setCpr('0987654321');
      flush();
      expect(calledTimes).toBe(1);
      sub.unsubscribe();
    }));

    it('clears xBreakTheGlassReason', fakeAsync(() => {
      let calledTimes = 0;
      const sub = service.xBreakTheGlassReason$.subscribe((reason) => {
        expect(reason).toBeUndefined();
        calledTimes++;
      });
      service.setCpr('0987654321');
      flush();
      expect(calledTimes).toBe(1);
      sub.unsubscribe();
    }));

    it('clears hasPrivatelyMarkedData', fakeAsync(() => {
      let calledTimes = 0;
      const sub = service.hasPrivatelyMarkedData$.subscribe((value) => {
        expect(value).toBeUndefined();
        calledTimes++;
      });
      service.setCpr('0987654321');
      flush();
      expect(calledTimes).toBe(1);
      sub.unsubscribe();
    }));
  });
});
