import {TestBed} from '@angular/core/testing';
import {UserContext, UserContextService} from './user-context.service';
import {HttpClientTestingModule, HttpTestingController} from '@angular/common/http/testing';
import {SorItem} from './sor.service';



describe('UserContextService', () => {
  let service: UserContextService;
  let httpTestController: HttpTestingController;
  const testBasePath = 'http://localhost:8280/graviditetsmappen';
  const testUserContext: UserContext = {
    commonName: 'Lars Larsen',
    organizationName: 'Test',
    eligibleRoles: [
      {
        authorizationCode: 'J0184',
        educationCode: '7170',
        educationType: 'Læge'
      },
      {
        authorizationCode: 'J0184',
        educationCode: '7170',
        educationType: 'Jordemoder'
      }
    ]
  };
  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    httpTestController = TestBed.inject(HttpTestingController);
    service = TestBed.inject(UserContextService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('should fetch userContext', (done) => {
    service.userContext$.subscribe(
    res => {
      expect(res).toBeDefined();
      done();
    }
    );
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(testUserContext);
    service.ngOnDestroy();
  });

  it('should select role when only one is eligible', (done) => {
    // copy test context and remove one role
    const singleRoleUser = Object.assign({}, testUserContext);
    singleRoleUser.eligibleRoles.pop();
    service.eligibleRoles$.subscribe(
      res => {
        expect(res).toBeDefined();
        done();
      }
    );
    service.selectedRole$.subscribe(
      res => {
        expect(res).toBeDefined();
        done();
      }
    );
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(singleRoleUser);
  });

  it('should select role', (done) => {
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(testUserContext);
    service.selectedRole$.subscribe(res =>  {
      expect(res).toBeDefined();
      done();
    }
    );
    service.selectRole(testUserContext.eligibleRoles[0]);
  });

  it('should select sor', (done) => {
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(testUserContext);
    service.selectedOrganizationSor$.subscribe(res =>  {
        expect(res).toBeDefined();
        done();
      }
    );
    const sorTest: SorItem = {name: 'Test', institutionOwnerSorId: 'test', sorId: 'test'};
    service.selectOrganizationSor(sorTest);
  });
});
