import {
  AfterViewChecked,
  AfterViewInit,
  Component,
  ContentChildren,
  ElementRef,
  forwardRef,
  Input,
  QueryList,
  ViewChild
} from '@angular/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import deepEqual from 'deep-equal';

@Component({
  selector: 'app-option',
  template: '<div></div>'
})
export class OptionComponent {
  @Input() label = '';
  @Input() value: any;

  constructor() { }
}

export const APP_TOGGLE_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => ToggleComponent),
  multi: true
};

@Component({
  selector: 'app-toggle',
  templateUrl: './toggle.component.html',
  styleUrls: ['./toggle.component.scss'],
  providers: [APP_TOGGLE_CONTROL_VALUE_ACCESSOR],
})
export class ToggleComponent implements AfterViewInit, ControlValueAccessor {
  @Input() label = '';
  @ContentChildren(OptionComponent, {descendants: true}) transcludedOptions: QueryList<OptionComponent>|undefined;

  public options: {label: string, value: any}[] = [];
  public innerModel: any;
  public optionWidth = '';
  @Input() public disabled = false;

  private dirtyAble = false;
  private onChangeCallback: (newValue: any[]) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor() { }

  ngAfterViewInit(): void {
    if (this.transcludedOptions) {
      this.optionWidth = `${100 / (this.transcludedOptions.length || 1)}%`;
      [...this.transcludedOptions.toArray()].forEach(x => {
        this.options.push({
          label: x.label,
          value: x.value
        });
      });
    }
  }

  get value(): any {
    return this.innerModel;
  }

  set value(value: any) {

    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value);
      }
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: any): void {
    this.value = newValue;
  }

  public isSelected(val: any): boolean {
    return deepEqual(val, this.value);
  }

  public makeDirtyAble(): void {
    this.dirtyAble = true;
  }

  public handleClick(newVal: any, event: MouseEvent): void {
    if (this.disabled) {
      event.preventDefault();
      return;
    }
    this.makeDirtyAble();
    this.value = newVal;
  }
}
