import {ActivityPlannedTime, DataCard} from '@nspop/gm-web-facade-api';

/**
 * Calculated as result of calculatePregnancyStartDate + fromWeek, and calculatePregnancyStartDate + toWeek respectively.
 */
export function calculateFromAndToWeek(dataCard: DataCard | undefined, fromWeek: number, toWeek: number): ActivityPlannedTime {
  const pregnancyStartDate = calculatePregnancyStartDate(dataCard);
  const from: Date | undefined = pregnancyStartDate ? getDatePlusXWeeks(pregnancyStartDate, fromWeek) : undefined;
  const to: Date | undefined = pregnancyStartDate ? getDatePlusXWeeks(pregnancyStartDate, toWeek)  : undefined;
  return {
    from,
    to,
  } as ActivityPlannedTime;
}

export function calculateWeekNumberFromPregnancyStart(dataCard: DataCard | undefined, week: Date | undefined): number | undefined {
  const pregnancyStartDate = calculatePregnancyStartDate(dataCard);
  if (!pregnancyStartDate || !week) { return; }

  const oneWeek = 7 * 24 * 60 * 60 * 1000; // days * hours * minutes * seconds * milliseconds

  return Math.round(Math.abs((+pregnancyStartDate - +week) / oneWeek));
}

function getDatePlusXDays(date: Date, days: number): Date {
  const newDate = new Date(date);
  newDate.setDate(date.getDate() + days);
  return newDate;
}

function getDatePlusXWeeks(date: Date, weeks: number): Date {
  return getDatePlusXDays(date, weeks * 7);
}

/**
 * Calculated first as expectedDateOfDeliveryFromPrenatalUltrasoundScan - 273 if it exists, and
 * expectedDateOfDeliveryFromLastPeriod.lastPeriod otherwise.
 * If neither exists, it returns undefined.
 */
export function calculatePregnancyStartDate(dataCard: DataCard | undefined): Date | undefined {
  // TODO: Clarification. Maybe this could be a field in the dataCard model so the calculation is consistent across WEB and BFF?
  let pregnancyStartDate: Date;
  const ultraSoundPrediction: Date | undefined = dataCard
    ?.expectedDateOfDeliverySection
    ?.expectedDateOfDeliveryFromPrenatalUltrasoundScan;

  const lastPeriod: Date | undefined = dataCard
    ?.expectedDateOfDeliverySection
    ?.expectedDateOfDeliveryFromLastPeriod
    ?.lastPeriod;

  if (ultraSoundPrediction) {
    const dateToSet: number = ultraSoundPrediction.getDate() - 273;
    pregnancyStartDate = new Date(ultraSoundPrediction);
    pregnancyStartDate.setDate(dateToSet);
  } else if (lastPeriod) {
    pregnancyStartDate = new Date(lastPeriod);
  } else {
    return undefined;
  }

  return pregnancyStartDate;
}
